from rest_framework.response import Response
from rest_framework import status
from django.shortcuts import get_object_or_404
from ellis_django_views.utils import error_messages, request_params
from ellis_django_views.views_abstract import (AbstractCreateView,
                                         AbstractRequestView,
                                         AbstractUpdateView,
                                         AbstractDeleteView)

# Utility function to check authorization
# Retrieves the authorization token from the request headers using a specified key from request params
def check_authorization(request, auth_token_key):
    auth_token = (request_params.get_request_header_param_value(
        request=request, key=auth_token_key))
    return auth_token

# Class to handle creating a new resource, inherites from AbstractCreateView 
class CreateViewImpl(AbstractCreateView):
    def post(self, request):
        try:
            # Check if the request is authorized
            if self.is_authorised(check_authorization(
                request=request, auth_token_key=self.auth_token_key)):
                # Deserialize request data
                serializer = self.serializer_model(
                    data=request.data)
                # Validate and save the serialized data
                if serializer.is_valid():
                    serializer.save()
                    return Response(
                        serializer.data,status=status.HTTP_200_OK)
                # Return validation errors if the data is invalid
                return Response(
                    {error_messages.ERROR : serializer.errors},
                    status=status.HTTP_400_BAD_REQUEST)
            # Return unauthorized status if authorization fails
            return Response(status=status.HTTP_401_UNAUTHORIZED)
        except Exception as e:
            # Return a specific error response in case of a ValueError
            return Response({error_messages.ERROR : str(e)},
                            status=status.HTTP_400_BAD_REQUEST)

# Class to handle retrieving resources, inherites from AbstractRequestView 
class RequestViewImpl(AbstractRequestView):
    def post(self, request):
        try:
            # Check if the request is authorized
            if self.is_authorised(check_authorization(
                request=request, auth_token_key=self.auth_token_key)):
                # Get primary key from request parameters
                pk = request_params.get_request_param_value(
                    request, request_params.PK)
                # Retrieve the object or return 404 if not found
                object = get_object_or_404(self.model, pk=pk)
                # Serialize the retrieved object
                serializer = self.serializer_model(instance=object)
                return Response(
                    serializer.data,status=status.HTTP_200_OK)
            # Return unauthorized status if authorization fails
            return Response(status=status.HTTP_401_UNAUTHORIZED)
        except ValueError as e:
            # Return a specific error response in case of a ValueError
            return Response({error_messages.ERROR : str(e)},
                            status=status.HTTP_400_BAD_REQUEST)

# Class to handle updating resources, inherites from AbstractUpdateView 
class UpdateViewImpl(AbstractUpdateView):
    def post(self, request):
        try:
            # Check if the request is authorized
            if self.is_authorised(check_authorization(
                request=request, auth_token_key=self.auth_token_key)):
                # Get primary key from request parameters
                pk = request_params.get_request_param_value(
                    request, request_params.PK)
                # Retrieve the object or return 404 if not found
                object = get_object_or_404(self.model, pk=pk)
                # Deserialize request data with the existing object instance
                serializer = self.serializer_model(
                    instance=object,data=request.data)
                # Validate and save the serialized data
                if serializer.is_valid():
                    serializer.save()
                    return Response(
                        serializer.data,status=status.HTTP_200_OK)
                # Return validation errors if the data is invalid
                return Response({
                    error_messages.ERROR : serializer.errors},
                    status=status.HTTP_400_BAD_REQUEST)
            # Return unauthorized status if authorization fails
            return Response(status=status.HTTP_401_UNAUTHORIZED)
        except ValueError as e:
            # Return a specific error response in case of a ValueError
            return Response({error_messages.ERROR : str(e)},
                            status=status.HTTP_400_BAD_REQUEST)

# Class to handle deleting resources, inherites from AbstractDeleteView 
class DeleteViewImpl(AbstractDeleteView):
    def post(self, request):
        try:
            # Check if the request is authorized
            if self.is_authorised(check_authorization(
                request=request, auth_token_key=self.auth_token_key)):
                # Get primary key from request parameters
                pk = request_params.get_request_param_value(
                    request, request_params.PK)
                # Retrieve the object or return 404 if not found
                object = get_object_or_404(self.model, pk=pk)
                # Delete the retrieved object
                object.delete()
                return Response(status=status.HTTP_200_OK)
            # Return unauthorized status if authorization fails
            return Response(status=status.HTTP_401_UNAUTHORIZED)
        except ValueError as e:
            # Return a specific error response in case of a ValueError
            return Response({error_messages.ERROR : str(e)},
                            status=status.HTTP_400_BAD_REQUEST)


ERROR = 'error'
MISSING_PARAMETER = 'Missing parameter: '
MISSING_HEADER_PARAMETER = 'Missing header parameter: '
INVALID_HTTPS_URL = 'Invalid url: check url begins with https://'
MISSING_URL = '{MISSING_PARAMETER}url'
FILE_NOT_SUBMITTED = 'No file was submitted.'


from abc import ABC, abstractmethod
from rest_framework.views import APIView
from ellis_django_views.utils import request_params

# Abstract base class for creating resources, inherits from ABC and APIView
class AbstractCreateView(ABC, APIView):
    # Placeholder for the serializer model, to be defined by subclasses
    serializer_model = None
    # Authorization token for request validation, obtained from request_params
    auth_token_key = request_params.AUTH_TOKEN
    @abstractmethod
    def is_authorised(self, auth_token):
        pass

# Abstract base class for requesting resources, inherits from ABC and APIView
class AbstractRequestView(ABC, APIView):
    # Placeholder for the model, to be defined by subclasses
    model = None
    # Placeholder for the serializer model, to be defined by subclasses
    serializer_model = None
    # Authorization token key used to extract Authorization token from request headers, defined in request_params
    auth_token_key = request_params.AUTH_TOKEN
    # Abstract method for checking authorization
    # Subclasses must implement this method to define their own authorization logic
    @abstractmethod
    def is_authorised(self, auth_token):
        pass

# Abstract base class for updating resources, inherits from ABC and APIView
class AbstractUpdateView(ABC, APIView):
    # Placeholder for the model, to be defined by subclasses
    model = None
    # Placeholder for the serializer model, to be defined by subclasses
    serializer_model = None
    # Authorization token key used to extract Authorization token from request headers, defined in request_params
    auth_token_key = request_params.AUTH_TOKEN
    # Abstract method for checking authorization
    # Subclasses must implement this method to define their own authorization logic
    @abstractmethod
    def is_authorised(self, auth_token):
        pass

# Abstract base class for deleting resources, inherits from ABC and APIView
class AbstractDeleteView(ABC, APIView):
    # Placeholder for the model, to be defined by subclasses
    model = None
    # Authorization token key used to extract Authorization token from request headers, defined in request_params
    auth_token_key = request_params.AUTH_TOKEN
    # Abstract method for checking authorization
    # Subclasses must implement this method to define their own authorization logic
    @abstractmethod
    def is_authorised(self, auth_token):
        pass


from django.conf import settings
from rest_framework import status
from django.test import TestCase, Client
from django.core.files.uploadedfile import SimpleUploadedFile
from ellis_django_views.utils import url_paths, request_params, error_messages
from ellis_django_views.tests.models import TestImageModel
from PIL import Image as PilImage
import tempfile
import shutil

class ViewsImplTests(TestCase):
    def setUp(self):
        self.client = Client()
        self.urls = url_paths
        self.test_image = 'test_image.png'
        self.test_updated_image = 'test_updated_image.png'
    
    def create_temp_image(self):
        """
        Helper method to create a temporary image file.
        """
        with tempfile.NamedTemporaryFile(
            suffix='.jpg', delete=False) as file:
            image = PilImage.new('RGB', (100, 100), 'white')
            image.save(file, format='PNG')
        return open(file.name, mode='rb')
    
    def upload_image(self, image_name):
        """
        Helper method to upload an image file.
        """
        image_file = self.create_temp_image()
        uploaded_file = SimpleUploadedFile(image_name, image_file.read(), content_type="image/png")
        return uploaded_file

    def test_image_POST_missing_auth_token(self):
        # Test case to check handling of missing authorization token in POST request
        # Simulates sending a POST request without an authorization token
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing auth token
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        response = self.client.post(path=f'/{self.urls.TEST_POST}',
                                    data=request)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(
            response.json()[error_messages.ERROR],
            f'{error_messages.MISSING_HEADER_PARAMETER}{request_params.AUTH_TOKEN}')


    def test_image_REQUEST_missing_auth_token(self):
        # Test case to check handling of missing authorization token in GET request
        # Simulates sending a GET request without an authorization token
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing auth token
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        image = TestImageModel.objects.get(pk=pk)
        request = {request_params.PK : pk}
        headers = {}
        response = self.client.post(
            path=f'/{self.urls.TEST_GET}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_400_BAD_REQUEST)
        self.assertEqual(
            response.json()[error_messages.ERROR],
            f'{error_messages.MISSING_HEADER_PARAMETER}{request_params.AUTH_TOKEN}')
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')

    def test_image_UPDATE_missing_auth_token(self):
        # Test case to check handling of missing authorization token in PUT request
        # Simulates sending a PUT request without an authorization token
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing auth token
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        uploaded_file =  self.upload_image(self.test_image)
        request = {request_params.PK : pk,
                   request_params.IMAGE : uploaded_file}
        image = TestImageModel.objects.get(pk=pk)
        headers = {}
        response = self.client.post(
            path=f'/{self.urls.TEST_PUT}', data=request, **headers)
        self.assertEqual(response.status_code,status.HTTP_400_BAD_REQUEST)
        self.assertEqual(
            response.json()[error_messages.ERROR],
            f'{error_messages.MISSING_HEADER_PARAMETER}{request_params.AUTH_TOKEN}')
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')

    def test_image_DELETE_missing_auth_token(self):
        # Test case to check handling of missing authorization token in DELETE request
        # Simulates sending a DELETE request without an authorization token
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing auth token
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        request = {request_params.PK : pk}
        headers = {}
        response = self.client.post(
            path=f'/{self.urls.TEST_DELETE}', data=request, **headers)
        self.assertEqual(response.status_code,status.HTTP_400_BAD_REQUEST)
        self.assertEqual(
            response.json()[error_messages.ERROR],
            f'{error_messages.MISSING_HEADER_PARAMETER}{request_params.AUTH_TOKEN}')
        images = TestImageModel.objects.all()
        self.assertEqual(len(images), 1)
        image = TestImageModel.objects.get(pk=pk)
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')

    def test_image_POST_missing_image(self):
        # Test case to check handling of missing image file in POST request
        # Simulates sending a POST request without an image file
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing image file
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}',
            content_type=request_params.CONTENT_TYPE, data={}, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json()[error_messages.ERROR],
                         {request_params.IMAGE:
                          [error_messages.FILE_NOT_SUBMITTED]})
        images = TestImageModel.objects.all()
        self.assertEquals(len(images), 0)

    def test_image_UPDATE_missing_image(self):
        # Test case to check handling of missing image file in PUT request
        # Simulates sending a PUT request without an image file
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing image file
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        request = {request_params.PK : pk}
        image = TestImageModel.objects.get(pk=pk)
        response = self.client.post(
            path=f'/{self.urls.TEST_PUT}', data=request, **headers)
        self.assertEqual(
            response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(
            response.json()[error_messages.ERROR],
            {request_params.IMAGE:[error_messages.FILE_NOT_SUBMITTED]})
        image = TestImageModel.objects.get(pk=pk)
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')

    def test_image_REQUEST_missing_pk(self):
        # Test case to check handling of missing primary key in GET request
        # Simulates sending a GET request without a primary key
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing primary key
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        image = TestImageModel.objects.get(pk=pk)
        request = {}
        response = self.client.post(
            path=f'/{self.urls.TEST_GET}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json()[error_messages.ERROR],
                         f'{error_messages.MISSING_PARAMETER}pk')
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')

    def test_image_UPDATE_missing_pk(self):
        # Test case to check handling of missing primary key in PUT request
        # Simulates sending a PUT request without a primary key
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing primary key
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        image = TestImageModel.objects.get(pk=pk)
        response = self.client.post(
            path=f'/{self.urls.TEST_PUT}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json()[error_messages.ERROR],
                         f'{error_messages.MISSING_PARAMETER}pk')
        image = TestImageModel.objects.get(pk=pk)
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')

    def test_image_DELETE_missing_pk(self):
        # Test case to check handling of missing primary key in DELETE request
        # Simulates sending a DELETE request without a primary key
        # Asserts that the response returns a 400 Bad Request status
        # and an appropriate error message about the missing primary key
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        pk = response.json()[request_params.ID]
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        request = {}
        response = self.client.post(
            path=f'/{self.urls.TEST_DELETE}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.json()[error_messages.ERROR],
                         f'{error_messages.MISSING_PARAMETER}pk')
        images = TestImageModel.objects.all()
        self.assertEqual(len(images), 1)
        images.first().delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')

    def test_image_CREATE_correct(self):
        # Test case to check correct creation of an image
        # Simulates sending a POST request with correct parameters
        # Asserts that the response returns a 200 OK status
        # and the image is created successfully in the database
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request,**headers)
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        image = TestImageModel.objects.get(pk=pk)
        self.assertEqual(image.image.url,
                         response.json()[request_params.IMAGE])
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')
    
    def test_image_REQUEST_correct(self):
        # Test case to check correct retrieval of an image
        # Simulates sending a GET request with correct parameters
        # Asserts that the response returns a 200 OK status
        # and the requested image is returned successfully
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        image = TestImageModel.objects.get(pk=pk)
        request = {request_params.PK : pk}
        response = self.client.post(
            path=f'/{self.urls.TEST_GET}', data=request, **headers)
        self.assertEqual(image.image.url,
                         response.json()[request_params.IMAGE])
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')

    def test_image_UPDATE_correct(self):
        # Test case to check correct updating of an image
        # Simulates sending a PUT request with correct parameters
        # Asserts that the response returns a 200 OK status
        # and the image is updated successfully in the database
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        uploaded_file = self.upload_image(self.test_updated_image)
        request = {request_params.PK : pk,
                   request_params.IMAGE : uploaded_file}
        image = TestImageModel.objects.get(pk=pk)
        response = self.client.post(
            path=f'/{self.urls.TEST_PUT}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        image = TestImageModel.objects.get(pk=pk)
        self.assertEqual(image.image.name,
                         f'{url_paths.IMAGES}{pk}/{self.test_updated_image}')
        image.delete()
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')
    
    def test_image_DELETE_correct(self):
        # Test case to check correct deletion of an image
        # Simulates sending a DELETE request with correct parameters
        # Asserts that the response returns a 200 OK status
        # and the image is deleted successfully from the database
        uploaded_file = self.upload_image(self.test_image)
        request = {request_params.IMAGE : uploaded_file}
        headers = {request_params.AUTH_TOKEN :
                   request_params.getBearer(request_params.AUTH_TOKEN)}
        response = self.client.post(
            path=f'/{self.urls.TEST_POST}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        pk = response.json()[request_params.ID]
        request = {request_params.PK : pk}
        response = self.client.post(
            path=f'/{self.urls.TEST_DELETE}', data=request, **headers)
        self.assertEqual(response.status_code,
                         status.HTTP_200_OK)
        images = TestImageModel.objects.all()
        self.assertEqual(len(images), 0)
        shutil.rmtree(f'{settings.MEDIA_ROOT}{url_paths.IMAGES}{pk}')


from ellis_django_views.tests.models import TestImageModel
from ellis_django_views.tests.serializers import TestImageSerializer
from ellis_django_views.views_implementation import (CreateViewImpl,
                                                     RequestViewImpl,
                                                     UpdateViewImpl,
                                                     DeleteViewImpl)

def is_authorised(self, auth_token):
    "Place holder for authorization"
    return True

class CreateTestImageView(CreateViewImpl):
    serializer_model = TestImageSerializer
    is_authorised = is_authorised
class RequestTestImageView(RequestViewImpl):
    model = TestImageModel
    serializer_model = TestImageSerializer
    is_authorised = is_authorised
class UpdateTestImageView(UpdateViewImpl):
    model = TestImageModel
    serializer_model = TestImageSerializer
    is_authorised = is_authorised
class DeleteTestImageView(DeleteViewImpl):
    model = TestImageModel
    is_authorised = is_authorised


POST = 'post/'
GET = 'get/'
PUT = 'put/'
DELETE = 'delete/'

TEST = 'test/'
IMAGES = 'images/'
TEST_POST = f'{TEST}{POST}'
TEST_GET = f'{TEST}{GET}'
TEST_PUT = f'{TEST}{PUT}'
TEST_DELETE = f'{TEST}{DELETE}'


# Generated by Django 4.2.13 on 2024-05-21 05:17

from django.db import migrations, models
import ellis_django_views.tests.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestImageModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(upload_to=ellis_django_views.tests.models.image_directory)),
                ('datetime', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



from rest_framework import serializers
from ellis_django_views.tests.models import TestImageModel

class TestImageSerializer(serializers.ModelSerializer):
    class Meta:
        model = TestImageModel
        fields = ['id','image', 'datetime']


from django.urls import path
from ellis_django_views.utils import url_paths, url_names
from ellis_django_views.tests.views import (CreateTestImageView,
                                            RequestTestImageView,
                                            UpdateTestImageView,
                                            DeleteTestImageView)

urlpatterns = [
    path(url_paths.POST, CreateTestImageView.as_view(), name=url_names.IMAGE_CREATE),
    path(url_paths.GET, RequestTestImageView.as_view(), name=url_names.IMAGE_REQUEST),
    path(url_paths.PUT, UpdateTestImageView.as_view(), name=url_names.IMAGE_UPDATE),
    path(url_paths.DELETE, DeleteTestImageView.as_view(), name=url_names.IMAGE_DELETE),
]


from django.db import models
from ellis_django_views.utils import url_paths

# Create your models here.

def image_directory(self, image_name):
    return f'testingEDV/{url_paths.IMAGES}{self.id}/{image_name}'

class TestImageModel(models.Model):
    image = models.ImageField(upload_to=image_directory)
    datetime = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        if self.id is None:
            saved_image = self.image
            self.image = None
            super(TestImageModel, self).save(*args, **kwargs)
            self.image = saved_image
            if 'force_insert' in kwargs:
                kwargs.pop('force_insert')
        super(TestImageModel, self).save(*args, **kwargs)


from ellis_django_views.utils import error_messages

AUTH_TOKEN = 'HTTP_AUTHORIZATION'
PK = 'pk'
URL = 'url'
ID = 'id'
IMAGE = 'image'
CONTENT_TYPE = 'application/json'

def getBearer(auth_token):
    return f'Token {auth_token}'

def get_request_param_value(request, key):
    value = request.data.get(key)
    if value is not None:
        return value
    raise ValueError(f'{error_messages.MISSING_PARAMETER}{key}')

def get_request_header_param_value(request, key):
    value = request.META.get(key)
    if value is not None:
        return value
    raise ValueError(f'{error_messages.MISSING_HEADER_PARAMETER}{key}')


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    include_package_data=True,
    name='ellis_django_views',
    version='0.232',
    packages=find_packages(),
    description='Generic views for Model View Serializer (MVS) pattern',
    author='Carl Victor Ellis',
    author_email='carl.ellis@hotmail.com.au',
    license='GNU',
    url='https://carlellis.io',
    long_description=open('README.md').read(),
    long_description_content_type= 'text/markdown',
    install_requires=[
        'django',
        'pillow',
        'djangorestframework',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    tests_require=[
        'pytest',
        'pytest-django',
    ],
    setup_requires=[
        'pytest-runner',
    ],
)